module PrepKit
  class Context
    def initialize(host, user, options = {})
      @vars     = {}
      @env      = environment(host, user, options)
      @memoized = {}
    end

    def task(name, &block)
      raise AssignError, name if @vars[name]

      @vars[name] = block
    end

    def load(filename)
      instance_eval open(filename, &:read), filename
    end

    def call(name)
      raise NotFoundError, name unless Proc === (block = @vars[name])

      Task.new(self).action &block
    end

    def sh(command)
      PrepKit.logger.info "sh: ".bold + command.underline

      status = @memoized.fetch(command) do
        @memoized[command] = :skip

        @env.exec!(command) == 0 ? :success : :failure
      end

      raise RuntimeError, command if status == :failure

      status
    end

    def test?(path, option)
      @env.test?(path, option) == 0
    end

    %i(upload download).each do |name|
      define_method(name) do |src, dst|
        PrepKit.logger.info "#{name}: ".bold + "#{src} #{dst}".underline

        @env.public_send("#{name}!", src, dst)
      end
    end

    private

    def environment(host, user, options)
      Remote.new(host, user, options) do |type, data|
        data = ((data = data.chomp) =~ /\n/) ? "\n" + data : data

        case type
        when :output
          PrepKit.logger.info  data.colorize(:green) unless data.empty?
        when :error
          PrepKit.logger.error data.colorize(:red)   unless data.empty?
        end
      end
    end
  end
end
