module PrepKit
  class Remote
    def initialize(host, user, options, &callback)
      @ssh, @scp = create_connection(host, user, options, &callback)
    end

    def exec!(command)
      @ssh.(command)
    end

    %i(upload! download!).each do |name|
      define_method(name) do |src, dst|
        @scp.() { |scp| scp.public_send(name, src, dst) }
      end
    end

    def test?(name, option = nil)
      exec!(%([ #{option} #{name} ]))
    end

    private

    def create_connection(host, user, options, &callback)
      connection = lambda do |&block|
        Net::SSH.start(host, user, options) do |ssh|
          block.(ssh)

          ssh.loop
        end
      end

      [create_ssh(connection, &callback), create_scp(connection)]
    end

    def create_ssh(connection, &callback)
      lambda do |command|
        status = nil

        connection.() do |ssh|
          channel = ssh.open_channel do |chan|
            chan.exec command do |ch, success|
              raise RuntimeError command unless success

              ch.on_data do |_, data|
                callback.(:output, data)
              end

              ch.on_extended_data do |_, type, data|
                callback.(:error, data)
              end

              ch.on_request('exit-status') do |_, data|
                status = data.read_long
              end
            end

            channel.wait
          end
        end

        status
      end
    end

    def create_scp(connection)
      -> (&callback) { connection.() { |ssh| callback.(ssh.scp) } }
    end
  end
end
